<?php
/*
 * ./lm_source/resources/general.php
 * Latest Member(s)
 *
 * @author Labradoodle-360
 * @version 1.0.5
 * @copyright Matthew Kerle 2012
*/

if (!defined('SMF'))
	die('Hacking attempt...');

/*
	- Returns a formatted array of the
		latest members.
	- Settings determined by $modSettings.
	- Return array is expected to be cached.
*/
function initializeLM()
{

	// Globalization
	global $modSettings, $user_info, $context;
	global $membergroups, $scripturl, $settings, $smcFunc;

	// Our Membergroups
	if (cache_get_data('lab_membergroups', 120) != null)
		$membergroups = cache_get_data('lab_membergroups', 120);
	else
		$membergroups = loadMemberGroups();

	// Empty Array.
	$latest_members = array();

	// Not set? Return.
	if (isset($modSettings['lm_enable']) == false)
		return $latest_members;
	// Set to Disabled? Return.
	elseif (isset($modSettings['lm_enable']) == true && $modSettings['lm_enable'] == 0)
		return $latest_members;
	// 0 Latest? Ha! Return.
	elseif (isset($modSettings['lm_num_latest']) == true && $modSettings['lm_num_latest'] == 0)
		return $latest_members;
	// Otherwise, visible to?
	elseif (isset($modSettings['lm_visible_to']) == true && !empty($modSettings['lm_visible_to']) && $modSettings['lm_visible_to'] != '-1')
	{
		$return = false; # begin at false.
		// Explode our array, get rid of guests.
		$visible_array = explode(',', str_replace(array('-1,', ',-1'), '', $modSettings['lm_visible_to']));

		// Begin our loop.
		for ($count = 0; $return !== true && $count !== (count($visible_array) + 1); $count++)
		{
			// Duplication of foreach();
			if (isset($visible_array[$count]))
				$value = $visible_array[$count];
			else
				continue;

			// Valid group? Come on.
			if (in_array($value, $user_info['groups']))
				$return = true;
			// Post group, possibly?
			elseif ($value == $user_info['id_post_group'])
				$return = true;
		}
		// Still nothing on close? Return.
		if ($return === false)
			return $latest_members;
	}

	// Start with our basic columns we'll need.
	$columns = 'id_member, real_name, is_activated';

	// We might need to add group id.
	if (!empty($modSettings['lm_group_colour']))
		$columns .= ', id_group, id_post_group';

	// (And | Or) registration date.
	if (!empty($modSettings['lm_date_registered']))
		$columns .= ', date_registered';

	// Then the actual query, "imploding" our columns ^^.
	$query_one = $smcFunc['db_query']('', '
		SELECT ' . $columns . '
		FROM {db_prefix}members
		WHERE is_activated = {int:is_activated}
		ORDER BY id_member DESC
		LIMIT {int:limit}',
		array(
			'is_activated' => 1,
			'limit' => (int) $modSettings['lm_num_latest']
		)
	);

	// Non-activated members? Let's try.
	if (isset($modSettings['lm_activated']) && !empty($modSettings['lm_activated']))
	{
		// What mode are we?
		switch ($modSettings['lm_activated'])
		{
			// Both 1 & 3 have possibilities for return.
			case 1 || 3:
				$query_two = $smcFunc['db_query']('', '
					SELECT ' . $columns . '
					FROM {db_prefix}members
					ORDER BY id_member DESC
					LIMIT {int:limit}',
					array(
						'limit' => (int) $modSettings['lm_num_latest']
					)
				);
			break;
			// Otherwise, we're not going anywhere but out.
			default:
				$query_two = '';
			break;
		}

		// Check for content.
		if (!empty($query_two))
		{
			// If we have some, process the second query.
			while ($row = $smcFunc['db_fetch_assoc']($query_two))
			{
				// Begin formatting our second array.
				$latest_members['all'][$row['id_member']] = array(
					'name' => $row['real_name'],
					'is_activated' => $row['is_activated'] == 1 ? 1 : 0
				);

				// We could have id group, depending on which columns were added based on settings.
				if (!empty($row['id_group']) || !empty($row['id_post_group']))
				{
					$all_identifier = !empty($row['id_group']) ? $row['id_group'] : $row['id_post_group'];
					$latest_members['all'][$row['id_member']]['group']['colour'] = array_key_exists($all_identifier, $membergroups['colours']) ? $membergroups['colours'][$all_identifier] : '';
				}
				// (And|Or) Date Registered, could be here.
				if (!empty($row['date_registered']))
					$latest_members['all'][$row['id_member']]['date_registered'] = timeformat($row['date_registered'], true, false); #formatted nicely.
			}

			// Free the results, before moving on.
			$smcFunc['db_free_result']($query_two);
		}
	}

	// Process the first query...
	while ($row = $smcFunc['db_fetch_assoc']($query_one))
	{
		// Begin formatting the first array.
		$latest_members['activated'][$row['id_member']] = array(
			'name' => $row['real_name'],
			'is_activated' => 1
		);

		// Again, two additional possibilities.
		if (!empty($row['id_group']) || !empty($row['id_post_group']))
		{
			$group_identifier = !empty($row['id_group']) ? $row['id_group'] : $row['id_post_group'];
			$latest_members['activated'][$row['id_member']]['group']['colour'] = array_key_exists($group_identifier, $membergroups['colours']) ? $membergroups['colours'][$group_identifier] : '';
		}
		if (!empty($row['date_registered']))
			$latest_members['activated'][$row['id_member']]['date_registered'] = timeformat($row['date_registered'], true, false);
	}
	// Free the first queries results.
	$smcFunc['db_free_result']($query_one);

	// Then cache our results to prevent overload.
	cache_put_data('lab_latest_members', $latest_members, 120);

	// Finally, return our array.
	return $latest_members;

}

/*
	- Determines whether or not the inputted parameter
		is an odd integer or not, and returns a boolean.
*/
function is_odd($int)
{
	if ($int % 2 == true)
		return true;
	else
		return false;
}

/*
	- Loads all SMF Membergroups into formatted arrays.
*/
function loadMemberGroups()
{

	// Globalize everything we'll need here...
	global $smcFunc, $txt;

	// We need a string...
	loadLanguage('lm_language/admin');

	// What is our request?
	$request = $smcFunc['db_query']('', '
		SELECT id_group, group_name, online_color, min_posts
		FROM {db_prefix}membergroups
		ORDER BY id_group'
	);

	// Do we have results, please?
	$membergroups = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		// ID Array
		$membergroups['id_array'][] = $row['id_group'];

		// Here are our Primary Membergroups
		if ($row['min_posts'] == '-1')
		{
			$membergroups['primary'][$row['id_group']] = array(
				'id' => $row['id_group'],
				'name' => $row['group_name'],
				'color' => $row['online_color']
			);
		}
		// Then Post-Based Membergroups.
		else
		{
			$membergroups['post_based'][$row['id_group']] = array(
				'id' => $row['id_group'],
				'name' => $row['group_name'],
				'color' => $row['online_color']
			);
		}
		// Colours...
		if (!empty($row['online_color']))
			$membergroups['colours'][$row['id_group']] = $smcFunc['strtolower']($row['online_color']);
	}

	// Free the results!!!
	$smcFunc['db_free_result']($request);

	// Two more key entries; Haha that's a pun. ;)
	$membergroups['id_array'][] = '0';
	$membergroups['id_array'][] = '-1';

	// Then add the array items for the entries.
	$membergroups['primary'] += array(
		'0' => array(
			'id' => '0',
			'name' => $txt['lm_regular_member'],
			'color' => ''
		),
		'-1' => array(
			'id' => '-1',
			'name' => $txt['lm_guest'],
			'color' => ''
		)
	);

	// Proudly cached for your convenience.
	cache_put_data('lab_membergroups', $membergroups, 120);

	// Return our findings.
	return $membergroups;

}